sbetter_server (sbroker v1.1.1)
ask and ask_r values for load balanacing processes with sbetter.
Link to this section Summary
Functions
Register the local process Pid with the server and sets the ask and ask_r values (such as sojourn times) as integer values AskValue and AskRValue.
Unregister the process Pid with the server.
Update the ask and ask_r with values (such as sojourn times) as integer values AskValue and AskRValue for Pid.
Link to this section Functions
Link to this function
register(Pid, AskValue, AskRValue)
Specs
register(Pid, AskValue, AskRValue) -> Result
when Pid :: pid(), AskValue :: integer(), AskRValue :: integer(), Result :: boolean().
Register the local process Pid with the server and sets the ask and ask_r values (such as sojourn times) as integer values AskValue and AskRValue.
true if the process is successfully registered, or false if already registered.
Link to this function
unregister(Pid)
Specs
unregister(Pid) -> true when Pid :: pid().
Unregister the process Pid with the server.
Pid.
Link to this function
update(Pid, AskValue, AskRValue)
Specs
update(Pid, AskValue, AskRValue) -> Result
when Pid :: pid(), AskValue :: integer(), AskRValue :: integer(), Result :: boolean().
Update the ask and ask_r with values (such as sojourn times) as integer values AskValue and AskRValue for Pid.
true if the values were updated, or false if Pid is not registered with the server.