sregulator_open_valve (sbroker v1.1.2)
Implements a simple valve with maximum capacity.
sregulator_open_value can be used as the sregulator_valve in a sregulator. It will provide a value that is always open up to a maximum capacity. Its argument, spec() is of the form: #{max => Max :: non_neg_integer() | infinity} % default: infinityMax is the maximum number of concurrent tasks the valve will allow to run (defaults to infinity). The valve remains open up to the maximum and then closes. It ignores any updates.
Summary
Types
Link to this type
spec/0
-type spec() :: #{max => Max :: non_neg_integer() | infinity}.